/*
 * All Rights Reserved.
 * Copyright (C) 1999-2005 Tsukuba Bunko.
 *
 * Licensed under the BSD License ("the License"); you may not use
 * this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.tsukuba-bunko.org/licenses/LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: FontManager.java,v 1.4 2005/11/09 10:39:37 ppoi Exp $
 */
package tsukuba_bunko.peko.resource;

import	java.awt.Font;
import	java.awt.GraphicsEnvironment;
import	java.awt.font.TextAttribute;

import java.io.File;
import	java.io.InputStream;

import	java.net.URL;

import	java.util.Iterator;
import	java.util.Map;
import	java.util.List;
import	java.util.Locale;
import	java.util.Set;

import	tsukuba_bunko.peko.Logger;

import	tsukuba_bunko.peko.resource.ResourceManager;


/**
 * tHg̊Ǘ񋟂܂B
 * @author	$Author: ppoi $
 * @version	$Revision: 1.4 $
 */
public class FontManager	{

	/**
	 * B̃CX^X
	 */
	private static FontManager	_instance = null;


	/**
	 * tHgLbV
	 */
	private Map	_fonts = null;

	/**
	 * gp\ȃtHgt@~Zbg
	 */
	private	Set	_fontFamilies = null;

	/**
	 * VKɍ쐬tHg
	 */
	private Map	_bundledFonts = null;


	/**
	 * <code>FontManager</code> ̃CX^X쐬܂B
	 */
	protected FontManager()
	{
		super();
	}


	/**
	 * <code>attributes</code> Ŏw肳ꂽtHg擾܂B
	 * @param	attributes	tHgɓKp鑮}bv
	 * @return	擾ꂽtHg
	 */
	public Font getFont( Map attributes )
	{
		Font	font = (Font)_fonts.get( attributes );
		if( font == null )	{
			String	family = (String)attributes.get( TextAttribute.FAMILY );
			Logger.debug( "[font] search font: \"" + family + "\"" );
			if( family == null )	{
				Logger.debug( "[font] not specified font family. using serif family (default)" );
				attributes.put( TextAttribute.FAMILY, "Serif" );
			}

			if( _bundledFonts.keySet().contains(family) )	{
				Logger.debug( "[font] using bundled font" );
				attributes.remove( TextAttribute.FAMILY );
				font = ((Font)_bundledFonts.get(family)).deriveFont( attributes );
				attributes.put( TextAttribute.FAMILY, family );
			}
			else if( _fontFamilies.contains(family) )	{
				Logger.debug( "[font] using system family" );
				font = new Font( attributes );
			}
			else	{
				Logger.error( "[font] invalid font family specified. family=" + family );
				Logger.debug( "[font] using serif family (default)" );
				attributes.remove( TextAttribute.FAMILY );
				font = new Font("Serif", 1, Font.PLAIN).deriveFont( attributes );
				attributes.put( TextAttribute.FAMILY, family );
			}
			_fonts.put( attributes, font );
		}
		return font;
	}

	/**
	 * 
	 */
	private void initialize()
	{
		ResourceManager	resources = ResourceManager.getInstance();

		_fontFamilies = new java.util.HashSet( 89 );

		Locale	locale = (Locale)resources.getResource( "peko.system.locale" );
		if( locale == null )	{
			locale = Locale.getDefault();
		}
		String[]	families = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames( locale );
		for( int i = 0; i < families.length; ++i )	{
			_fontFamilies.add( families[i] );
		}

		boolean	isJ5 = false;
		try	{
			Font.class.getMethod( "createFont", new Class[]{int.class, File.class} );
			isJ5 = true;
		}
		catch( Exception e )	{
			//ignore
		}

		_bundledFonts = new java.util.HashMap( 89 );
		List	additional = (List)resources.getResource( "peko.system.fonts", true );
		if( (additional != null) && !additional.isEmpty() )	{
			InputStream	fontStream = null;
			URL	basedir = resources.getLocationResources().getMiscDirecotryURL();
			String	fontPath = null;
			Iterator	itr = additional.iterator();
			while( itr.hasNext() )	{
				try	{
					fontPath = (String)itr.next();
					if( (fontPath == null) || (fontPath.length() == 0) )	{
						continue;
					}
					Logger.debug( "[font] loading font \"" + fontPath + "\"." );
					Font	font = null;
					URL	fontURL = new URL( basedir, fontPath );
					String	protocol = fontURL.getProtocol();
					if( isJ5 && "file".equals(protocol) )	{
						File	fontFile = new File( fontURL.getFile() );
						font = Font.createFont( Font.TRUETYPE_FONT, fontFile );
					}
					else	{
						fontStream = fontURL.openStream();
						font = Font.createFont(Font.TRUETYPE_FONT, fontStream );
					}
					String	fontName = fontPath;
					int	extDelim = fontName.lastIndexOf( '.' );
					if( extDelim != -1 )	{
						fontName = fontName.substring( 0, extDelim );
					}
					_bundledFonts.put( fontName, font );
					_bundledFonts.put( (String)font.getAttributes().get(TextAttribute.FAMILY), font );	//݊̂
				}
				catch( Exception e )	{
					Logger.warn( "[font] fail to load font \"" + fontPath + "\".", e );
				}
				finally	{
					if( fontStream != null )	{
						try	{
							fontStream.close();
						}
						catch( Exception e )	{
							Logger.error( "[font] fail to close font stream. \"" + fontPath + "\".", e );
						}
						fontStream = null;
					}
				}
			}
		}

		_fonts = new java.util.WeakHashMap( 89 );
	}



	/**
	 * B <code>FontManager</code> CX^X擾܂B
	 * @return	B <code>FontManager</code> CX^X
	 */
	public static FontManager getInstance()
	{
		if( _instance == null )	{
			synchronized( FontManager.class )	{
				if( _instance == null )	{
					_instance = new FontManager();
					_instance.initialize();
				}
			}
		}
		return _instance;
	}
}
